<?php

App::uses('Component', 'Controller');

class TranslationComponent extends Component {

	protected $_controller = null;

	public function __construct(ComponentCollection $collection, $settings = []) {
		parent::__construct($collection, $settings);
		$this->settings = $settings;
	}

/**
 * Sets a reference to the calling controller from within the component.
 *
 * @see Component::initialize()
 */
	public function initialize(Controller $controller) {
		parent::initialize($controller);
		$this->_controller = $controller;
	}

	public function adminTranslationFormFields() {
		$Page = $this->_controller->{$this->_controller->modelClass};
		$fields = $Page->schema();

		$fields = array_intersect_key(
			$fields,
			array_flip($this->settings['fields'])
		);

		$languages = Translation::getLocales(true);

		$data = [];
		foreach ($languages as $language) {
			foreach ($fields as $key => $value) {
				$data[$language][$Page->alias . '.__i18n.' . $language . '.content.' . $key] = $value;
				$data[$language][$Page->alias . '.__i18n.' . $language . '.id'] = array(
					'type' => 'integer'
				);
			}

			// Include menu fields for translation.
			if (!empty($this->settings['menu'])) {
				$data[$language]['Menu.__i18n.' . $language . '.id'] = array(
					'type' => 'integer'
				);
				$data[$language]['Menu.__i18n.' . $language . '.content.name'] = array(
					'type' => 'string',
					'label' => __('Menu')
				);
			}

			// Include meta data fields for translation.
			if (!empty($this->settings['meta'])) {
				$data[$language]['MetaData.__i18n.' . $language . '.id'] = array(
					'type' => 'integer'
				);
				$data[$language]['MetaData.__i18n.' . $language . '.content.title'] = array(
					'type' => 'string',
					'label' => __('Meta Title')
				);
				$data[$language]['MetaData.__i18n.' . $language . '.content.description'] = array(
					'type' => 'string',
					'label' => __('Meta Description')
				);
			}
		}

		$this->_controller->set('translationFields', $data);
		return;
	}

}
